package Lobby;
use strict;
use IO::Socket::INET;
use IO::Select;
use Guest;
use Room;
use Msg;
use Option;
use Log;

#--------------------
sub new{
	my($proto, %args) = @_;
	my $class = ref($proto) || $proto;
	my $this  = {};
	$this->{fh} = undef;
	$this->{sel} = undef;
	$this->{buf} = undef;
	$this->{msg} = undef;
	$this->{g_no} = undef;
	$this->{r_no} = undef;
	$this->{guests} = [];
	$this->{rooms} = [];
	$this->{nr} = undef;
	$this->{ng} = undef;
	$this->{opt} = undef;
	$this->{log} = undef;
	bless($this, $class);

	foreach(keys %args){
		$this->{$_} = $args{$_};
	}
	$this->{g_no} = 1;
	$this->{r_no} = 1;
	$this->{guests} = ();
	$this->{rooms} = ();
	$this->{nr} = 0;
	$this->{ng} = 0;
	$this->{msg} = new Msg();
	$this->{opt} = new Option();
	$this->{log} = new Log();

	return $this;
}

#--------------------
sub open{
	my $this = shift;
	my $port = shift;
	$this->{fh} = new IO::Socket::INET(
		LocalAddr => $this->{opt}->addr
		, LocalPort =>  $this->{opt}->port
		, Proto => "tcp"
		, Type => SOCK_STREAM
		, Reuse => 1
		, Listen => 5
		) or die "Couldn't get server socket....\n$!\n";

	$this->{sel} = new IO::Select($this->{fh});

	print "Wating...\n";
}

#--------------------
sub close{
	my $this = shift;
	close($this->{fh});
	print "Closed.\n" ;
}

#--------------------
sub work{
	my $this = shift;
	my($fh, $new, $addr, $port, $rv, $g);
	while(1){
		foreach $fh($this->{sel}->can_read()){
			$g = ${$this->{guests}}{$fh};
			if($fh == $this->{fh}){
				#guest limit verification
				$this->ng >= $this->{opt}->g_limit and next;

				$new = $this->{fh}->accept();
				$new < 0 and die "Accept error!!";
		
				$this->{sel}->add($new);

				$new->send($this->{msg}->welcome . "\n");
				$new->send($this->{msg}->name_req . "\n");

				printf("Got a connection from %s, %s.\n", $this->peerinfo($new));
				$this->{log}->write(sprintf "Got a connection from %s, %s.\n", $this->peerinfo($new));
			}
			else{
				$rv = $fh->recv($this->{buf}, 1024);
				if(defined($rv) and length($this->{buf})){

					unless($g){		#If the guest does not exists
						if($this->is_valid_name($fh, $this->{buf})){$this->add_guest($fh);}
						else{$fh->send($this->{msg}->name_req . "\n");	}
						next;
					}

					#If the guest exists
					$this->{opt}->tap and printf("Received from %s: %s\n", $g->name, $this->buf);
					$this->{opt}->log and $this->{log}->write(sprintf "Received from %s: %s\n"
						, $g->name, $this->buf);

					if($this->{buf} =~ /^\// or $g->cmd){
						$this->serv_cmd($g, $this->{buf});
					}
					else{
						$g->room and $g->room->talk2all($g->name . ": " . $this->{buf} . "\n");
					}
				}
				else{
					$g and $this->del_guest($g);
					$this->disconn($fh);
				}
			}
		}
	}
}

#--------------------
sub peerinfo{
	my $this = shift;
	my $fh = shift;
	my($port, $iaddr) = unpack_sockaddr_in($fh->peername());
	my $addr = inet_ntoa($iaddr);
	return ($addr, $port);
}

#--------------------
sub disconn{
	my $this = shift;
	my $fh = shift;
	$this->{sel}->remove($fh);
	close($fh);
	printf("A client has been disconnected.\n");
	$this->{opt}->log and $this->{log}->write("A client has been disconnected.\n");
}

#--------------------
sub is_valid_name{
	my $this = shift;
	my($fh, $name) = @_;

	if($this->is_there_none_word_character($name)){
		$fh->send($this->{msg}->name_none . "\n");
		return 0;
	}

	$name =~ s/\s+//g;
	if(length($name) > 12 or length($name) < 4){
		$fh->send(sprintf($this->{msg}->name_len, $name) . "\n");
		return 0;
	}

	#Is there already a guest who has same name?
	if($this->find_guest($name)){
		$fh->send($this->{msg}->name_same($name) . "\n");
		return 0;
	}
	return 1;
}

#--------------------
sub is_there_none_word_character{
	my $this = shift;
	my $line = shift;
	my @chrs = split(//, $line);
	my $k;
	foreach(@chrs){
		$k = ord($_);
		if($k >= 0 and $k <= 127){
			if($k < 48 or ($k > 57 and $k < 65) or ($k > 90 and $k < 97) or ($k > 122)){
				return 1;
			}
		}
	}
	return 0;
}

#--------------------
sub add_guest{
	my $this = shift;
	my $fh = shift;

	my $g = new Guest(fh => $fh, no => $this->{g_no}++, name => $this->{buf});
	$g->fh->send($this->{msg}->name_chg($g->name) . "\n");
	
	${$this->{guests}}{$g->fh} = $g;
	$g->fh->send($this->{msg}->help("lob", $g) . "\n");
	$this->{ng}++;
	printf("A guest(%s) has been added.\n", $g->name);
	$this->print_info();
}

#--------------------
sub del_guest{
	my $this = shift;
	my $g = shift;

	#If guest has a room
	$g->room and $g->room->del_guest($g);

	#if there's no one in the room
	if($g->room){
		$g->room->guests or $this->del_room($g->room);
	}

	delete ${$this->{guests}}{$g->fh};
	$this->{ng}--;
	printf("A guest(%s) has been deleted.\n", $g->name);
	$this->print_info();
}

#--------------------
sub add_room{
	my $this = shift;
	my $g = shift;

	my %hash = $g->args;
	my $r = new Room(no => $this->{r_no}++, owner => $g
		, subj => $hash{'subj'}, pw => $hash{'pw'}, limit => $hash{'limit'});
	$g->room($r);
	$r->add_guest($g);

	${$this->{rooms}}{$r->no} = $r;
	$this->{nr}++;
	printf "pw = %s\n", $r->pw;
	printf("A room(%s) has been added.\n", $r->no);
	$this->print_info();
}

#--------------------
sub del_room{
	my $this = shift;
	my $r = shift;

	delete ${$this->{rooms}}{$r->no};
	$this->{nr}--;
	printf("A room(%s) has been deleted.\n", $r->no);
	$this->print_info();
}

#--------------------
sub print_info{
	my $this = shift;

	print "\n-------------------\n";
	my @fh_arr = keys(%{$this->{guests}});
	printf "Number of guest in lobby = %s\n", $this->{ng};

	my $i = 1;
	while(my($fh, $g) = each(%{$this->{guests}})){
		printf("%d: %s\n", $i++, $g->name);
	}

	print "-------------------\n";
	my @r_no_arr = keys(%{$this->{guests}});
	printf "Number of room in lobby = %s\n", $this->{nr};

	my $i = 1;
	while(my($r_no, $r) = each(%{$this->{rooms}})){
		printf("%d: %s - %s\n", $i++, $r->no, $r->owner->name, $r->subj);
	}
}

#--------------------
sub talk2all{
	my $this = shift;
	my $line = shift;

	while(my($fh, $g) = each(%{$this->{guests}})){
		$g->fh->send($line);
	}
}

#--------------------
sub find_guest{
	my $this = shift;
	my $name = shift;

	my @arr = keys %{$this->{guests}};
	my $g;
	foreach(@arr){
		$g = ${$this->{guests}}{$_};
		$g->name eq $name and return $g;
	}
}

#--------------------
sub cancel_cmd{
	my $this = shift;
	my $g = shift;

	$g->fh->send($this->{msg}->cmd_can($g->cmd) . "\n");
	$g->delete_cmd($g);
}

#--------------------
sub serv_cmd_X{
	my $this = shift;
	my $g= shift;

	if($g->room){
		$g->room->del_guest($g);
		#if there's no one in the room
		$g->room->guests or $this->del_room($g->room);
		$g->room(undef);
		return;
	}

#	$g->fh->send($this->{msg}->lob_exit . "\n");
#	$g and $this->del_guest($g);
#	$this->disconn($g->fh);
}

#--------------------
sub serv_cmd_N{
	my $this = shift;
	my $g= shift;
	my $cmd= shift;
	my @args = @_;

	unless(@args){
		$g->fh->send($this->{msg}->usage($cmd) . "\n");
		return;
	}

	if($this->is_valid_name($g->fh, $args[0])){
		my $name = $g->name($args[0]);
		$g->fh->send($this->{msg}->name_chg($name) . "\n");
	}
}

#--------------------
sub serv_cmd_O{
	my $this = shift;
	my $g= shift;
	my $cmd= shift;

	$g->fh->send($this->{msg}->room_subj . "\n");
	$g->fh->send($this->{msg}->cancel . "\n");
	$g->cmd($cmd);
	$g->step(2);
}

#--------------------
sub serv_cmd_J{
	my $this = shift;
	my $g= shift;
	my $cmd= shift;
	my @args = @_;

	unless(@args){
		$g->fh->send($this->{msg}->usage($cmd) . "\n");
		return;
	}

	my $r_no = $args[0];
	my $r = ${$this->{rooms}}{$r_no};

	#if the room does not exists
	$r or return;

	#if the room is private
	if($r->pw){
		$g->fh->send($this->{msg}->pw_req($r_no) . "\n");
		$g->fh->send($this->{msg}->cancel . "\n");
		$g->cmd($cmd);
		$g->add_arg("r_no", $r_no);
		$g->step(2);
		return;
	}
	
	#Enter the room
	$g->room($r);
	$r->add_guest($g);
}

#--------------------
sub serv_cmd_F{
	my $this = shift;
	my $g= shift;
	my $cmd= shift;
	my @args = @_;

	#Wrong command
	unless(@args){
		$g->fh->send($this->{msg}->usage($cmd) . "\n");
		return;
	}

	#find guest
	my $tgt_name = $args[0];
	my $found = $this->find_guest($tgt_name);
	if($found){
		if($found->room){
			$g->fh->send($this->{msg}->find("room", $found->name, $found->room->no) . "\n");
		}
		else{
			$g->fh->send($this->{msg}->find("lob", $found->name) . "\n");
		}

		return;
	}
	$g->fh->send($this->{msg}->find("not", $tgt_name) . "\n");
}

#--------------------
sub serv_cmd_I{
	my $this = shift;
	my $g= shift;
	my $cmd= shift;
	my @args = @_;

	#Wrong command
	unless(@args){
		$g->fh->send($this->{msg}->usage($cmd) . "\n");
		return;
	}

	#find guest
	my $tgt_name = $args[0];
	my $found = $this->find_guest($tgt_name);
	if($found){
		$found->fh->send($this->{msg}->invite("invite", $g) . "\n");
		$g->fh->send($this->{msg}->invite("confirm", $found) . "\n");

		return;
	}
	$g->fh->send($this->{msg}->find("not", $tgt_name) . "\n");
}

#--------------------
sub serv_cmd_K{
	my $this = shift;
	my $g= shift;
	my $cmd= shift;
	my @args = @_;

	#Wrong command
	unless(@args){
		$g->fh->send($this->{msg}->usage($cmd) . "\n");
		return;
	}

	#find guest
	my $tgt_name = $args[0];
	my $found = $g->room->find_guest($tgt_name);
	if($found){
		$g->room->del_guest($found, "kick");
		$g->room(undef);
		return;
	}
	$g->fh->send($this->{msg}->find("not_room", $tgt_name, $g->room->no) . "\n");
}

#--------------------
sub serv_cmd_P{
	my $this = shift;
	my $g= shift;
	my $cmd= shift;

	$g->fh->send($this->{msg}->pw_new . "\n");
	$g->fh->send($this->{msg}->cancel . "\n");
	$g->cmd($cmd);
	$g->step(2);
}

#--------------------
sub serv_cmd_S{
	my $this = shift;
	my $g= shift;
	my $cmd= shift;

	$g->fh->send($this->{msg}->subj_new . "\n");
	$g->fh->send($this->{msg}->cancel . "\n");
	$g->cmd($cmd);
	$g->step(2);
}

#--------------------
sub serv_cmd_LM{
	my $this = shift;
	my $g= shift;
	my $cmd= shift;

	$g->fh->send($this->{msg}->limit_new . "\n");
	$g->fh->send($this->{msg}->cancel . "\n");
	$g->cmd($cmd);
	$g->step(2);
}

#--------------------
sub serv_cmd{
	my $this = shift;
	my($g, $line) = @_;

#	$g->print_info();

	if($g->cmd and $line eq "0"){
		$this->cancel_cmd($g);
		return;
	}

	if($g->step == 2){
		$this->serv_cmd2($g, $line);
		return;
	}
	elsif($g->step == 3){
		$this->serv_cmd3($g, $line);
		return;
	}
	elsif($g->step == 4){
		$this->serv_cmd4($g, $line);
		return;
	}
	elsif($g->step == 5){
		$this->serv_cmd5($g, $line);
		return;
	}
	elsif($g->step == 6){
		$this->serv_cmd6($g, $line);
		return;
	}

	my(@args, $cmd);
	@args = split(/\s+/, $line);
	$cmd = shift(@args);
	$cmd = uc($cmd);

	printf "Got command '%s' from %s.\n", $cmd, $g->name;
	if($cmd eq "/X"){
		$g->room or return;		#room only cmd
		$this->serv_cmd_X($g)
	}
	elsif($cmd eq "/H"){
		if($g->room){$g->fh->send($this->{msg}->help("room", $g) . "\n");}
		else{$g->fh->send($this->{msg}->help("lob", $g) . "\n");}
	}
	elsif($cmd eq "/N"){
		$this->serv_cmd_N($g, $cmd, @args)
	}
	elsif($cmd eq "/O"){		#subj
		if($this->nr >= $this->{opt}->r_limit){
			$g->fh->send($this->{msg}->r_limit . "\n");
			return;
		}
		$this->serv_cmd_O($g, $cmd)
	}
	elsif($cmd eq "/J"){
		$g->room and return;		#lobby only cmd
		$this->serv_cmd_J($g, $cmd, @args)
	}
	elsif($cmd eq "/L"){
		$g->room and return;		#lobby only cmd
		$g->fh->send($this->room_list . "\n");
	}
	elsif($cmd eq "/A"){
		$g->fh->send($this->guest_list . "\n");
	}
	elsif($cmd eq "/F"){
		$this->serv_cmd_F($g, $cmd, @args)
	}
	elsif($cmd eq "/I"){
		$g->room or return;		#room only cmd
		if($g->name eq $args[0]){		#can't invite himself.
			$g->fh->send($this->{msg}->self . "\n");
			return;
		}
		$this->serv_cmd_I($g, $cmd, @args)
	}
	elsif($cmd eq "/R"){
		$g->room or return;		#room only cmd
		$g->fh->send($g->room->info . "\n");
	}
	elsif($cmd eq "/K"){
		$g->room or return;		#room only cmd
		if($g->name eq $args[0]){		# can't kickout himself.
			$g->fh->send($this->{msg}->self . "\n");
			return;
		}

		unless($g == $g->room->owner){		#room owner only
			$g->fh->send($this->{msg}->owner_only . "\n");
			return;
		}

		$this->serv_cmd_K($g, $cmd, @args)
	}
	elsif($cmd eq "/P"){
		$g->room or return;		#room only cmd

		unless($g == $g->room->owner){		#room owner only
			$g->fh->send($this->{msg}->owner_only . "\n");
			return;
		}

		$this->serv_cmd_P($g, $cmd)
	}
	elsif($cmd eq "/S"){
		$g->room or return;		#room only cmd

		unless($g == $g->room->owner){		#room owner only
			$g->fh->send($this->{msg}->owner_only . "\n");
			return;
		}

		$this->serv_cmd_S($g, $cmd)
	}
	elsif($cmd eq "/LM"){
		$g->room or return;		#room only cmd

		unless($g == $g->room->owner){		#room owner only
			$g->fh->send($this->{msg}->owner_only . "\n");
			return;
		}

		$this->serv_cmd_LM($g, $cmd)
	}
	elsif($cmd eq "/D"){
		$g->room or return;		#room only cmd

		unless($g == $g->room->owner){		#room owner only
			$g->fh->send($this->{msg}->owner_only . "\n");
			return;
		}
		
		unless($g->room->pw){
			$g->fh->send($this->{msg}->already_public . "\n");
			return;
		}

		$g->room->pw("");
		$g->fh->send($this->{msg}->pw_del . "\n");
	}
}

#--------------------
sub serv_cmd2{
	my $this = shift;
	my($g, $arg) = @_;

	my $cmd = $g->cmd;
	printf "Got arg '%s' of command '%s' from %s at step %s.\n"
		, $arg, $cmd, $g->name, $g->step;

	if($cmd eq "/O"){		#subj and limit
		$g->add_arg("subj", $arg);

		$g->fh->send($this->{msg}->room_limit . "\n");
		$g->fh->send($this->{msg}->cancel . "\n");
		$g->cmd($cmd);
		$g->step($g->step + 1);
	}
	if($cmd eq "/J"){		#password verification
		my %hash = $g->args;
		my $r_no = $hash{'r_no'};
		my $r = ${$this->{rooms}}{$r_no};
		
		#If room does not exists(not needed...)
		$r or return;

		if($arg ne $r->pw){
			$g->fh->send($this->{msg}->pw_wrong . "\n");
			$g->delete_cmd;
			return;
		}
	
		#Enter the room
		$g->room($r);
		$r->add_guest($g);
		$g->delete_cmd;
	}
	if($cmd eq "/P"){		#new password
		$g->fh->send($this->{msg}->pw_confirm . "\n");
		$g->fh->send($this->{msg}->cancel . "\n");
		$g->step($g->step + 1);
		$g->add_arg("pw", $arg);
	}
	if($cmd eq "/S"){		#new subj
		printf "room arg = %s\n", $arg;
		$g->room->subj($arg);
		printf "room subj = %s\n", $g->room->subj;
		$g->fh->send($this->{msg}->subj_chg($g->room->subj) . "\n");
		$g->delete_cmd;
	}
	if($cmd eq "/LM"){		#new limit
		$g->room->limit($arg);
		$g->fh->send($this->{msg}->limit_chg($g->room->limit) . "\n");
		$g->delete_cmd;
	}
}

#--------------------
sub serv_cmd3{
	my $this = shift;
	my($g, $arg) = @_;

	my $cmd = $g->cmd;
	printf "Got arg '%s' of command '%s' from %s at step %s.\n"
		, $arg, $cmd, $g->name, $g->step;

	if($cmd eq "/O"){		#limit and type of the room
		$g->add_arg("limit", $arg);

		$g->fh->send($this->{msg}->room_private . "\n");
		$g->fh->send($this->{msg}->yes_no_cancel . "\n");
		$g->step($g->step + 1);
	}
	if($cmd eq "/P"){		#password
		my %hash = $g->args;
		if($arg ne $hash{'pw'}){
			$g->fh->send($this->{msg}->pw_diff . "\n");
			return;
		}
		$g->room->pw($hash{'pw'});
		$g->fh->send($this->{msg}->pw_chg($g->room->pw) . "\n");
		$g->delete_cmd;
	}
}

#--------------------
sub serv_cmd4{
	my $this = shift;
	my($g, $arg) = @_;

	my $cmd = $g->cmd;
	printf "Got arg '%s' of command '%s' from %s at step %s.\n"
		, $arg, $cmd, $g->name, $g->step;

	if($cmd eq "/O"){
		if(uc($arg) eq "Y"){
			$g->fh->send($this->{msg}->room_pw . "\n");
			$g->fh->send($this->{msg}->cancel . "\n");
			$g->step($g->step + 1);
			return;
		}
		$this->add_room($g);
		$g->delete_cmd;
	}
}

#--------------------
sub serv_cmd5{
	my $this = shift;
	my($g, $arg) = @_;

	my $cmd = $g->cmd;
	printf "Got arg '%s' of command '%s' from %s at step %s.\n"
		, $arg, $cmd, $g->name, $g->step;

	if($cmd eq "/O"){		#password
		$g->fh->send($this->{msg}->pw_confirm . "\n");
		$g->fh->send($this->{msg}->cancel . "\n");
		$g->step($g->step + 1);
		$g->add_arg("pw", $arg);
	}
}

#--------------------
sub serv_cmd6{
	my $this = shift;
	my($g, $arg) = @_;

	my $cmd = $g->cmd;
	printf "Got arg '%s' of command '%s' from %s at step %s.\n"
		, $arg, $cmd, $g->name, $g->step;

	if($cmd eq "/O"){		#password fail or make new room
		my %hash = $g->args;
		if($arg ne $hash{'pw'}){
			$g->fh->send($this->{msg}->pw_diff . "\n");
			$g->add_arg("pw", "");
			$g->step(4);
			$this->serv_cmd($g, "Y");
			return;
		}
		$this->add_room($g);
		$g->delete_cmd;
	}
}


#--------------------
sub room_list{
	my $this = shift;

#Room: 34 / Participant: 56 / All: 343
#--------------------------------------------------------------------------------
#1	Private	(3/12)	[Edge]	About soccer.
#4	Public	(4/12)	[Steve]	I love movies.
#6	Public	(6/12)	[Kelly]	For programmers.
#--------------------------------------------------------------------------------

	my @arr = keys %{$this->{rooms}};
	my($res, $r, $guest_in_room, $no, $type, $info, $owner_name, $subj);
	foreach(sort {$b <=> $a} @arr){
		$r = ${$this->{rooms}}{$_};
		
		$no = $_;
		$type = $r->pw ? $this->{msg}->list_info("private") : $this->{msg}->list_info("public");
		$info = sprintf "(%s/%s)", $r->ng, $r->limit;
		$owner_name = $r->owner->name;
		$subj = $r->subj;

		$res .= "\n" . pack("A6A10A6A20A38", $no, $type, $info, sprintf("[%s]", $owner_name), $subj);

		$guest_in_room += $r->ng;
	}
	$res = sprintf "%s%s\n%s", $this->{msg}->line, $res, $this->{msg}->line;
	my $list_info = sprintf "%s: %d/ %s: %d/ %s: %d"
							, $this->{msg}->list_info("room"), $this->{nr}
							, $this->{msg}->list_info("participant"), $guest_in_room
							, $this->{msg}->list_info("all"), $this->{ng};
	return "\n" . $list_info . "\n" . $res;
}

#--------------------
sub guest_list{
	my $this = shift;

#Total number of guest: 457
#--------------------------------------------------------------------------------
#1	[Edge]		5	For soccer
#4	[Soccer]	3	I love movie
#6	[nobody]	7	we don't know anything...
#--------------------------------------------------------------------------------

	my @arr = keys %{$this->{guests}};
	my($res, $g, $g_no, $g_name, $r, $r_no, $r_subj);
	foreach(sort {$b <=> $a} @arr){
		$g = ${$this->{guests}}{$_};
		$r = $g->room;
		
		$g_no = $g->no;
		$g_name = $g->name;

		if($r){
			$r_no = $r->no;
			$r_subj = $r->subj;
		}
		else{
			$r_no = 0;
			$r_subj = "";
		}

		$res .= "\n" . pack("A6A20A6A46", $g_no, sprintf("[%s]", $g_name) , $r_no, $r_subj);
	}
	$res = sprintf "%s%s\n%s", $this->{msg}->line, $res, $this->{msg}->line;
	my $list_info = sprintf "%s: %d", $this->{msg}->list_info("all"), $this->{ng};
	return "\n" . $list_info . "\n" . $res;
}

#--------------------
sub fh{
	my $this = shift;
	if(@_){$this->{fh} = shift;}
	return $this->{fh};
}

#--------------------
sub sel{
	my $this = shift;
	if(@_){$this->{sel} = shift;}
	return $this->{sel};
}

#--------------------
sub buf{
	my $this = shift;
	if(@_){$this->{buf} = shift;}
	return $this->{buf};
}

#--------------------
sub g_no{
	my $this = shift;
	if(@_){$this->{g_no} = shift;}
	return $this->{g_no};
}

#--------------------
sub msg{
	my $this = shift;
	if(@_){$this->{msg} = shift;}
	return $this->{msg};
}

#--------------------
sub guests{
	my $this = shift;
	if(@_){%{$this->{guests}} = %_}
	return %{$this->{guests}};
}

#--------------------
sub ng{
	my $this = shift;
	if(@_){$this->{ng} = shift;}
	return $this->{ng};
}

#--------------------
sub nr{
	my $this = shift;
	if(@_){$this->{nr} = shift;}
	return $this->{nr};
}

#--------------------
1;
