package Room;
use strict;
use Msg;

#--------------------
sub new{
	my($proto, %args) = @_;
	my $class = ref($proto) || $proto;
	my $this  = {};
	$this->{no} = undef;
	$this->{owner} = undef;
	$this->{subj} = undef;
	$this->{pw} = undef;
	$this->{limit} = undef;
	$this->{guests} = [];
	$this->{ng} = undef;
	bless($this, $class);

	foreach(keys %args){
		$this->{$_} = $args{$_};
	}
	$this->{guests} = ();
	$this->{ng} = 0;

	#verify title
	length($this->{subj}) < 4 and $this->{subj} = sprintf "%s's room."
		, $this->{owner}->name;
	$this->{subj} = str_cuth($this->{subj}, 40);

	#verify limit
	($this->{limit} < 4 or $this->{limit} > 12) and $this->{limit} = 12;

	$this->{msg} = new Msg();

	return $this;
}

#--------------------
sub del_guest{
	my $this = shift;
	my $g = shift;
	my $flag = shift;

	for(my $i = 0; $i < @{$this->{guests}}; $i++){
		if(${$this->{guests}}[$i] == $g){
			splice(@{$this->{guests}}, $i, 1);
			$this->{ng}--;
			printf("A guest(%s) has been deleted from room %s.\n", $g->name, $this->{no});

			if($flag eq "kick"){$this->talk2all($this->{msg}->room_kick($g->name) . "\n");}
			else{$this->talk2all($this->{msg}->room_left($g->name) . "\n");}
			$g->fh->send($this->{msg}->help("lob", $g) . "\n");

			#if there's no one in the room
			$this->guests or return ;
			
			if($g == $this->{owner}){		#replace owner
				$this->{owner} = ${$this->{guests}}[0];

				$this->talk2all($this->{msg}->replace_owner($this->{owner}->name) . "\n");
			}

			return;
		}
	}
}

#--------------------
sub add_guest{
	my $this = shift;
	my $g = shift;

	push @{$this->{guests}}, $g;

	my($name, $subj) = ($this->{owner}->name, $this->{subj});
	$g->fh->send($this->{msg}->help("room", $g) . "\n");
	
	$this->talk2all($this->{msg}->room_enter($g->name) . "\n");

	$this->{ng}++;
	printf("A guest(%s) has been added to room %s.\n"
		, $g->name, $this->{no});
	$this->print_info;
}

#--------------------
sub print_info{
	return;
	my $this = shift;

	print "\n-------------------\n";
	printf "Room info of room %s\n", $this->{no};
	printf "No %s\n", $this->{no};
	printf "Number of the guest %s\n", $this->{ng};
	printf "Owner %s\n", $this->{owner}->name;
	printf "Subj %s\n", $this->{subj};
	printf "PW %s\n", $this->{pw};
	printf "Limit %s\n", $this->{limit};

	print "-------------------\n";
	printf "Number of guest in room = %s\n", $this->{ng};

	my $i = 1;
	foreach(@{$this->{guests}}){
		printf("%d: %s\n", $i++, $_->name);
	}
}

#--------------------
sub find_guest{
	my $this = shift;
	my $name = shift;

	foreach(@{$this->{guests}}){
		$_->name eq $name and return $_;
	}
}

#--------------------
sub talk2all{
	my $this = shift;
	my $line = shift;

	foreach(@{$this->{guests}}){
		$_->fh->send($line);
	}
}

#--------------------
sub info{
	my $this = shift;

#Total number of guest: 3
#--------------------------------------------------------------------------------
#Room owner: Edge
#Room no: 23
#Room subject: for programmers
#Room type: Public
#Room limit: 12

#1	[Edge]
#2	[Soccer]
#3	[nobody]
#--------------------------------------------------------------------------------

	my @g_names;
	foreach(@{$this->{guests}}){
		push(@g_names, $_->name);
	}

	my $res = "";
	my $i = 1;
	foreach(sort @g_names){
		$res .= sprintf "\n%d: %s", $i++, $_;
	}
	$res =~ s/^\n//;

	return $this->{msg}->room_info($this, $res);
}

#--------------------
sub no{
	my $this = shift;
	if(@_){$this->{no} = shift;}
	return $this->{no};
}

#--------------------
sub owner{
	my $this = shift;
	if(@_){$this->{owner} = shift;}
	return $this->{owner};
}

#--------------------
sub subj{
	my $this = shift;

	if(@_){
		my $subj = shift;
		#verify title
		length($this->{subj}) < 4 and $subj = sprintf "%s's room.", $this->{owner}->name;
		$subj = str_cuth($subj, 40);
		$this->{subj} = $subj;
	}
	return $this->{subj};
}

#--------------------
sub pw{
	my $this = shift;
	if(@_){$this->{pw} = shift;}
	return $this->{pw};
}

#--------------------
sub limit{
	my $this = shift;
	if(@_){
		my $limit = shift;
		($this->{limit} < 4 or $this->{limit} > 12) and $limit = 12;
		$this->{limit} = $limit;
	}
	return $this->{limit};
}

#--------------------
sub guests{
	my $this = shift;
	if(@_){@{$this->{guests}} = @_}
	return @{$this->{guests}};
}

#--------------------
sub ng{
	my $this = shift;
	if(@_){$this->{ng} = shift;}
	return $this->{ng};
}

#--------------------
sub str_cuth{		#ѱ ڸ Լ
	my($line, $size) = @_;

	my $res = "";
	my @chrs = split(//, $line);
	for(my $i = 0; $i < $size; $i++){
		if(ord($chrs[$i] <= 127)){$res .= $chrs[$i];}
		else{
			if($i >= $size - 1){last;}
			$res .= $chrs[$i++] . $chrs[$i];
		}
	}
 	return $res;
}

#--------------------
1;
